<?php
namespace Teknisa\PLA\Util;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

class validacaoCardapio extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $connection;
    protected $environment;
    protected $utilListas;
    public function __construct($crudService, $parameterbag, $environment, $connection, $listas){
        $this->environment = $environment;
        $this->connection = $connection;
        $this->utilListas = $listas;
    }
    
    private function trataTipoBloqueio($tipoBloqueio, &$bloqRequisicao, &$bloqLibCard, &$bloqAprovCard, &$bloqAprovMatriz, &$bloqPrevCons){
        switch ($tipoBloqueio) {
            case 'REQUISICAO':
                $bloqRequisicao = 'S';
                break;
            case 'LIBCARD':
                $bloqLibCard = 'S';
                break;
            case 'APROVCARD':
                $bloqAprovCard = 'S';
                break;
            case 'APROVMATRIZ':
                $bloqAprovMatriz = 'S';
                break;
            case 'PREVCONS':
                $bloqPrevCons = 'S';
                break;
            default:
                return false;
            
        }
        return true;
    }
    
    /* Recebe uma row contendo um escopo de serviços e verifica a situação de validação dos cardápios dos mesmos
    * CDGRUPFILI - vazio ou um código de agrupamento de filial
    * FILIAIS - array de string, string concatenada ou "T"
    * GRUPSERV - array de string, string concatenada ou "T"
    * SERVICOS - array  de array associativo ['CDFILIAL' e 'CDSERVFILI'] ou "T"
    * VDSERVATIV - bool indica se considera apenas serviços habilitados para requisição, retiradas
    * VDDESCENTR - bool indica se considera apenas serviços que dependem de liberação
    * VDSERVNAOHOSP - bool indica se deve desconsiderar serviços hospitalares
    * DTINI - Data inicial (dia/mes/ano) do período considerado na operação
    * DTFIN - Data final (dia/mes/ano) do período considerado na operação
    * TIPOBLOQUEIO - indica qual o tipo de bloqueio deve ser consistido nessa validação, valores possíveis:
    * - REQUISICAO  -> bloqueio de requisição
    * - LIBCARD     -> bloqueio de liberação de cardápios
    * - APROVCARD   -> bloqueio de aprovação do cardápio pela filial
    * - APROVMATRIZ -> bloqueio de aprovação do cardápio pela matriz
    * - PREVCONS    -> bloqueio de previsão de consumo
    * UTILIZALISTASERVICO - Indica que a geração da lista de serviço deve ocorrer obrigatoriamente, independente da validação do cardápio
    */
    
    public function verificaValidacaoListaServicos(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $cdGrupFili = $row['CDGRUPFILI'];
            $filiais = $row["FILIAIS"];
            $grupServ = $row["GRUPSERV"];
            $servicos = $row["SERVICOS"];
            $vdServAtiv = $row["VDSERVATIV"];
            $vdDescentr = $row["VDDESCENTR"];
            $vdServNaoHosp = $row["VDSERVNAOHOSP"];
            $dtIni = $row['DTINI'];
            $dtFin = $row['DTFIN'];
            $tipoBloqueio = $row['TIPOBLOQUEIO'];
            $bloqRequisicao = 'N';
            $bloqLibCard = 'N';
            $bloqAprovCard = 'N';
            $bloqAprovMatriz = 'N';
            $bloqPrevCons = 'N';
            $utilizaListaServico = $row['UTILIZALISTASERVICO'];
            
            $resultado = [
                "SERVICOS_A_IGNORAR" => [],
                "NRLISTA_SERVICO" => ""
            ];
            
            if(!$this->trataTipoBloqueio($tipoBloqueio, $bloqRequisicao, $bloqLibCard, 
                                         $bloqAprovCard, $bloqAprovMatriz, $bloqPrevCons)){
                $response->addDataSet(new DataSet('resultado', $resultado));
                return;
            }
            
            //Verifica se há uso da validação do cardápio (e justificaria uso da validação)
            $sql = "
            SELECT COUNT(1) AS USOS_VALIDA_CARD 
              FROM PARAMPLANEJA
             WHERE IDUTLVALCARD = 'S'
            ";
            $prepared = $this->connection->prepare($sql);
            $prepared->execute();
            $uso = $prepared->fetch(\PDO::FETCH_ASSOC);
            
            $nrlista = "";
            if(($uso["USOS_VALIDA_CARD"] > 0) || $utilizaListaServico) {
                //Se utilizar, cria a lista de filiais e serviços conforme parâmetros de uso
                $nrlista = $this->utilListas->retornaListaFilialServico($cdGrupFili, $filiais, $grupServ, $servicos, 
                                                                        $vdServAtiv, $vdDescentr, $vdServNaoHosp);
            }
            if($utilizaListaServico){
                $resultado['NRLISTA_SERVICO'] = $nrlista;
            }
            
            //Se não utiliza validação de cardápio, retorna a lista vazia
            if($uso["USOS_VALIDA_CARD"] == 0){
                $response->addDataSet(new DataSet('resultado', $resultado));
                return;
            }
            
            $sql = "
            SELECT S.CDFILIAL, S.CDSERVFILI, S.NMSERVICO
              FROM LISTA L
                   INNER JOIN SERVICO S ON S.CDFILIAL = SUBSTR(L.CDLISTA, 1, 4) AND
                                           S.CDSERVFILI = SUBSTR(L.CDLISTA, 6, 10)
                   INNER JOIN PARAMPLANEJA P ON P.CDFILIAL = S.CDFILIAL
             WHERE L.NRLISTA = :NRLISTA
               AND P.IDUTLVALCARD = 'S'
               AND ((:IDBLOQREQVALCARD = 'N') OR ((:IDBLOQREQVALCARD = 'S' AND P.IDBLOQREQVALCARD = 'S')))
               AND ((:IDBLOQLIBCAVALCARD = 'N') OR ((:IDBLOQLIBCAVALCARD = 'S' AND P.IDBLOQLIBCAVALCARD = 'S')))
               AND ((:IDBLOQAPRCAVALCARD = 'N') OR ((:IDBLOQAPRCAVALCARD = 'S' AND P.IDBLOQAPRCAVALCARD = 'S')))
               AND ((:IDBLOQAPMATVALCARD = 'N') OR ((:IDBLOQAPMATVALCARD = 'S' AND P.IDBLOQAPMATVALCARD = 'S')))
               AND ((:IDBLOQPREVCONVALCARD = 'N') OR ((:IDBLOQPREVCONVALCARD = 'S' AND P.IDBLOQPREVCONVALCARD = 'S')))
               AND EXISTS (SELECT 1 
                             FROM CARDADIA C
                                  LEFT JOIN CARDAMES CM ON CM.CDFILIAL = C.CDFILIAL AND
                                                           CM.CDSERVFILI = C.CDSERVFILI AND
                                                           CM.DTMESCARDAPIO = C.DTMESCARD
                            WHERE C.CDFILIAL = S.CDFILIAL
                              AND C.CDSERVFILI = S.CDSERVFILI
                              AND C.DTCARDAPIO BETWEEN TRUNC(TO_DATE(:DTINI, 'DD/MM/YYYY'), 'MM') AND LAST_DAY(TO_DATE(:DTFIN, 'DD/MM/YYYY'))
                              AND NVL(CM.IDVALIDADO, 'N') = 'N'
                              AND C.QTCOMESTICAR > 0)
            ";
            $prepared = $this->connection->prepare($sql);
            $params = [
                "NRLISTA" => $nrlista,
                "DTINI" => $dtIni,
                "DTFIN" => $dtFin,
                "IDBLOQREQVALCARD" => $bloqRequisicao,
                "IDBLOQLIBCAVALCARD" => $bloqLibCard,
                "IDBLOQAPRCAVALCARD" => $bloqAprovCard,
                "IDBLOQAPMATVALCARD" => $bloqAprovMatriz,
                "IDBLOQPREVCONVALCARD" => $bloqPrevCons
            ];
            $prepared->execute($params);
            $servicosAIgnorar = $prepared->fetchAll();
            if(count($servicosAIgnorar) > 0){
                
                //Remove da lista os serviços a ser ignorados
                if($utilizaListaServico){
                    $sql = "DELETE FROM LISTA L
                             WHERE L.NRLISTA = :NRLISTA
                               AND L.CDLISTA = :CDLISTA";
                    $prepared = $this->connection->prepare($sql);
                    foreach($servicosAIgnorar as $serv){
                        $params = [
                            "NRLISTA" => $nrlista,
                            "CDLISTA" => $serv["CDFILIAL"] . '_' . $serv["CDSERVFILI"]  
                        ];
                        $prepared->execute($params);
                    }
                }
                
                $resultado["SERVICOS_A_IGNORAR"] = $servicosAIgnorar;
            }
            if(!$utilizaListaServico)
                $this->utilListas->deleteLista($nrlista);
            $response->addDataSet(new DataSet('resultado', $resultado));
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
        
    }
}
